@@ -2,7 +2,7 @@
  * xlat.c	Translate strings.  This is the first version of xlat
  * 		incorporated to RADIUS
  *
- * Version:	$Id: xlat.c,v 1.134 2007/12/17 14:57:20 aland Exp $
+ * Version:	$Id: xlat.c,v 1.135 2008/03/16 17:59:29 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: xlat.c,v 1.134 2007/12/17 14:57:20 aland Exp $")
+RCSID("$Id: xlat.c,v 1.135 2008/03/16 17:59:29 aland Exp $")
 
 #include	<freeradius-devel/radiusd.h>
 #include	<freeradius-devel/rad_assert.h>
@@ -345,6 +345,15 @@ static size_t xlat_packet(void *instance, REQUEST *request,
 				return strlen(out);
 				break;
 
+			case PW_MODULE_RETURN_CODE:
+				localvp.attribute = da->attr;
+
+				/*
+				 *	See modcall.c for a bit of a hack.
+				 */
+				localvp.vp_integer = request->simul_max;
+				break;
+
 			default:
 				return 0; /* not found */
 				break;
